﻿using System;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using System.Windows.Media.Imaging;

namespace SeriTile
{
    public class AnimatedTile : ITile
    {
        private class TileFrame
        {
            public TileFrame(ITile tile, float frameTime)
            {
                Tile = tile;
                FrameTime = frameTime;
            }
            public ITile Tile { get; private set; }
            public float FrameTime { get; private set; }//Time between frames
        }

        System.Collections.Generic.List<TileFrame> m_tiles = new System.Collections.Generic.List<TileFrame>();
        TileFrame m_curFrame;
        float m_curTime;//Time between frames
        int m_curIndex;
        Random rand = new Random();
        public delegate void OnFinishedHandler(AnimatedTile sender, EventArgs eventArgs);
        public OnFinishedHandler OnFinished;
        public string Name { get; private set; }
        public bool Loop { get; private set; }
        public bool Finished { get; private set; }

        public AnimatedTile(string name, bool loop)
        {
            Name = name;
            Loop = loop;
        }

        public ITile Tile
        {
            get
            {
                return m_curFrame.Tile;
            }
        }

        public AnimatedTile Clone()
        {
            AnimatedTile newTile = new AnimatedTile(Name, Loop);
            foreach (TileFrame tile in m_tiles)
            {
                newTile.addFrame(tile.Tile, tile.FrameTime); 
            }

            //Randomize
            newTile.m_curIndex = rand.Next(m_tiles.Count);
            newTile.m_curFrame = m_tiles[m_curIndex];
            newTile.m_curTime = (float)(rand.NextDouble() * m_curFrame.FrameTime);

            return newTile;
        }

        public void addFrame(ITile tile, float frameTime)
        {
            TileFrame frame = new TileFrame(tile, frameTime);
            if (m_curFrame == null)
                m_curFrame = frame;
            m_tiles.Add(frame);
        }

        public void Start()
        {
            if (!Loop)
            {
                Finished = false;
                m_curTime = 0;
                m_curIndex = 0;
            }
        }

        public void update(float deltaTime)
        {
            if (Finished)
                return;

            m_curTime += deltaTime;

            if (m_curTime >= m_curFrame.FrameTime)
            {
                m_curTime = 0;
                m_curIndex++;
 
                if (m_curIndex >= m_tiles.Count)
                {
                    if (!Loop)
                    {
                        Finished = true;
                        m_curIndex = m_tiles.Count - 1;
                        if (OnFinished != null)
                            OnFinished(this, new EventArgs());
                    }
                    else
                    {
                        m_curIndex = 0;

                    }
                }
                m_curFrame = m_tiles[m_curIndex];
            }
        }

        public override Rect Rect 
        {
            get
            {
                return Tile.Rect;
            }
        }

        public override Point Offset 
        {
            get
            {
                return Tile.Offset;
            }
        }

        public override WriteableBitmap Image {
            get 
            {
                return Tile.Image;
            }

        }
        public override WriteableBitmapExtensions.BlendMode BlendMode 
        {
            get
            {
                return Tile.BlendMode;
            }
            internal set
            {
                Tile.BlendMode = value;
            }
        }

        public override void Blit(WriteableBitmap bitmap, Point blitPoint)
        {
            bitmap.Blit(blitPoint, Image, Rect, Colors.White, BlendMode);
        }

        public override void Blit(WriteableBitmap bitmap, Point blitPoint, Color color)
        {
            bitmap.Blit(blitPoint, Image, Rect, color, BlendMode);
        }

    }
}
