﻿using System;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using System.Collections.Generic;

namespace SeriTile
{
    public class Pathing
    {
        static private Pathing m_singleton;
        static private Dictionary<int, PathMapTile> m_tilesSeen = new Dictionary<int, PathMapTile>();
        static private List<PathMapTile> m_pool = new List<PathMapTile>();
        static private Queue<PathMapTile> m_tilesToCheck = new Queue<PathMapTile>();
        private int m_size;
        private TileMap m_map;
        int m_tilesCreated;//debug - tiles ever created

        private Pathing()
        {
        }

        static public Pathing GetSingleton()
        {
            if (m_singleton == null)
                m_singleton = new Pathing();
            return m_singleton;
        }

        /// <summary>
        /// Gets all regions (tiles) within the certain inclusive distance of the starting position
        /// </summary>
        /// <param name="map">The map to look at</param>
        /// <param name="startX">The starting X position</param>
        /// <param name="startY">The starting Y position</param>
        /// <param name="near">The inclusive near distance</param>
        /// <param name="far">The inclusive far distance</param>
        /// <param name="path">The output region list</param>
        public void GetRegionsWithinDistance(TileMap map, int startX, int startY, int near, int far, out List<IntPoint> result)
        {
            m_size = map.Width;
            m_map = map;
            m_tilesCreated = 0;
            result = new List<IntPoint>();
            Clear();

            PathMapTile startNode = GetTile(startX, startY);
            startNode.cost = 0;
            m_tilesToCheck.Enqueue(startNode);

            while (m_tilesToCheck.Count > 0)
            {
                PathMapTile tile = m_tilesToCheck.Dequeue();
                tile.onOpen = true;
                if (tile.cost <= far)
                {
                    if (tile.cost >= near)
                    {
                        result.Add(new SeriTile.IntPoint(tile.x, tile.y));
                    }
                    List<IntPoint> connections;
                    map.getConnectingPoints(null, new IntPoint(tile.x, tile.y), out connections);
                    foreach (IntPoint point in connections)
                    {
                        PathMapTile newTile = GetTile(point.X, point.Y);
                        if (newTile.onOpen == false)
                        {
                            newTile.cost = tile.cost + 1;
                            m_tilesToCheck.Enqueue(newTile);
                        }
                        //Else we already checked it
                    }
                }
                //Else it's too large so don't check any children
            }
        }

        void Clear()//adds all tiles in m_tilesToCheck and m_tilesSeen back to the pool
        {
            while (m_tilesToCheck.Count > 0)
            {
                PathMapTile cur = m_tilesToCheck.Dequeue();
            }

            foreach (PathMapTile tile in m_tilesSeen.Values)
            {
                m_pool.Add(tile);
            }
            m_tilesSeen.Clear();
        }

        PathMapTile GetTile(int x, int y)//gets a tile from the pool or creates one if there aren't any
        {
            PathMapTile tile;
            bool found = m_tilesSeen.TryGetValue(x + y * m_size, out tile);

            if (found)
            {
                return tile;
            }
            else
            {
                if (m_pool.Count == 0)
                {
                    tile = new PathMapTile();
                    m_tilesCreated++;
                }
                else
                {
                    tile = m_pool[m_pool.Count - 1];
                    m_pool.RemoveAt(m_pool.Count - 1);
                }
                tile.x = x;
                tile.y = y;
                tile.onOpen = false;//OnOpen is use to check if we've looked at this tile
                m_tilesSeen.Add(x + y * m_size, tile);
                return tile;
            }
        }
    }
}
