﻿using System;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using System.Xml.Linq;
using System.Linq;

namespace SeriTile
{
    public class Sprite
    {
        public System.Collections.Generic.Dictionary<string, AnimatedTile> m_animations = new System.Collections.Generic.Dictionary<string, AnimatedTile>();
        AnimatedTile m_animation;
        public delegate void OnAnimationFinishedHandler(Sprite sender, EventArgs eventArgs);
        public OnAnimationFinishedHandler OnAnimationFinished;

        public Sprite(string resourcePath, string fileName, TileSetManager tilesetManager)
        {
            XDocument doc = XDocument.Load(resourcePath + fileName);
            LoadSprite(doc.Root.Element("Sprite"), resourcePath, tilesetManager);
        }

        public Sprite(XElement element, string resourcePath, TileSetManager tilesetManager)
        {
            LoadSprite(element, resourcePath, tilesetManager);
        }

        private void LoadSprite(XElement element, string resourcePath, TileSetManager tilesetManager)
        {
            string tilesetName = (string)element.Attribute("TileSetName");
            string tilesetFile = (string)element.Attribute("TileSet");
            tilesetManager.LoadTileSet(tilesetName, resourcePath, tilesetFile);
            TileSet tileset = tilesetManager.GetTileSet(tilesetName);

            foreach (XElement animElement in element.Elements("Animation"))
            {
                string animName = (string)animElement.Attribute("Name");
                string tile = (string)animElement.Attribute("Tile");
                addAnimation(animName, (SeriTile.AnimatedTile)tileset.getTile(tile));

            }
        }

        public AnimatedTile createAnimation(string name, bool loop)
        {
            AnimatedTile animation = new AnimatedTile(name, loop);
            m_animations.Add(name, animation);
            return animation;
        }

        public void addAnimation(string name, AnimatedTile animatedTile)
        {
            AnimatedTile animation = animatedTile.Clone();
            animation.OnFinished += OnTileAnimationFinished;
            m_animations.Add(name, animation);
        }

        public void OnTileAnimationFinished(AnimatedTile tile, EventArgs args)
        {
            if(OnAnimationFinished != null)
                OnAnimationFinished(this, new EventArgs());
        }

        public void changeAnimation(string name)
        {
            if(OnAnimationFinished != null)
                OnAnimationFinished(this, new EventArgs());

            m_animation = m_animations[name];
            m_animation.Start();
        }

        public void update(float deltaTime)
        {
            m_animation.update(deltaTime);
        }

        public ITile Tile
        {
            get
            {
                return m_animation.Tile;
            }
        }
    }
}
