﻿using System;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using System.Xml.Linq;
using System.Linq;

namespace SeriTile
{
    public class TileSetManager
    {
        System.Collections.Generic.Dictionary<string, TileSet> m_tilesets = new System.Collections.Generic.Dictionary<string,TileSet>();
        static TileSetManager m_singleton = null;

        private TileSetManager()
        {
        }

        public static TileSetManager GetSingleton()
        {
            if (m_singleton == null)
                m_singleton = new TileSetManager();

            return m_singleton;
        }

        public TileSet GetTileSet(string name)
        {
            if (m_tilesets.ContainsKey(name))
                return m_tilesets[name];

            return null;
        }

        public TileSet LoadTileSet(string name, string imageFile, int height, int width, int baseHeight, System.Windows.Media.Imaging.WriteableBitmapExtensions.BlendMode blendMode)
        {
            if (m_tilesets.ContainsKey(name))
                return GetTileSet(name);
            TileSet tileSet = new TileSet(imageFile, height, width, baseHeight, blendMode);
            m_tilesets.Add(name, tileSet);
            return tileSet;
        }


        public TileSet LoadTileSet(string name, string resourcePath, string fileName)
        {
            if (m_tilesets.ContainsKey(name))
                return GetTileSet(name);
            TileSet tileSet = new TileSet(resourcePath, fileName);
            m_tilesets.Add(name, tileSet);
            return tileSet;
        }


        public TileSet LoadTileSet(string tsxFile)
        {
            XDocument doc = XDocument.Load(tsxFile);
            string name = (string)doc.Root.Attribute("name");

            if (m_tilesets.ContainsKey(name))
                return GetTileSet(name);

            TileSet tileSet = new TileSet(doc);
            m_tilesets.Add(name, tileSet);

            return tileSet;
        }

        public void update(float deltaTime)
        {
            foreach (System.Collections.Generic.KeyValuePair<string, TileSet> tileset in m_tilesets)
                tileset.Value.update(deltaTime);
        }
    }
}
